using System;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace ViewState
{	
	public class BasePage : System.Web.UI.Page
	{
		private void Page_Load(object sender, System.EventArgs e)
		{			
		}

		/// <summary>
		/// Pekryt metody pro ukldn ViewState do zvolenho loit.
		/// </summary>
		/// <param name="viewStateBag">Objekt ve kterm je obsah ViewState</param>
		protected override void SavePageStateToPersistenceMedium(object viewStateBag) 
		{
			// zjisteni typu uloziste ViewState z konfigurace
			EViewStateStore configuredStore = Configuration.ViewStateStore;						
			BasePersistor usedPersistor;
			int objectExpirationTimeout = Configuration.ObjectExpiration;
			// podle typu uloziste vytvorime instanci prislusne implementacni tridy
			switch (configuredStore) 
			{
				case EViewStateStore.Cache:
					// uloziste implementovano v pametove cache					
					usedPersistor = new CachePersistor(Cache, objectExpirationTimeout);
					break;
				case EViewStateStore.Database:
					// uloziste implementovano v databazi
					string configuredConnectionString = Configuration.ConnectionString;
					usedPersistor = new DatabasePersistor(configuredConnectionString, objectExpirationTimeout);
					break;
				case EViewStateStore.File:
					// uloziste implementovano v adresari na disku
					string configuredDirectory = Configuration.Directory;
					usedPersistor = new FilePersistor(configuredDirectory, objectExpirationTimeout);
					break;
				default:
					base.SavePageStateToPersistenceMedium(viewStateBag);
					return;					
			}			
			try 
			{
				// jedinecny identifikator viewState
				Guid viewStateID = Guid.Empty;
				// zjistime jestli ve strance je zaregistrovan klic k ulozenemu ViewState
				if (Request.Form["__VIEWSTATE_KEY"] != null)
				{					
					// pokud je ve skrytem poli na strance ulozen ViewState
					viewStateID = new Guid(Request.Form["__VIEWSTATE_KEY"]);
				}				
				else 
				{
					// jinak vytvorime novy identifikator
					viewStateID = Guid.NewGuid();
				}
				// ulozime ViewState do uloziste 
				usedPersistor.SaveViewState(
					viewStateID,
					viewStateBag);		
				
				// ve verzi .NET Frameworku 1.0 neni automaticky registrovano skryte pole pro ViewState
				// a ViewState pak nefunguje; v 1.1 jiz neni treba
				if (Environment.Version.Major == 1 && Environment.Version.Minor == 0) 
				{
					// zaregistrujeme skryte polozky na klienta - __VIEWSTATE musi byt pritomna, jinak cela ta sranda nefunguje...
					RegisterHiddenField("__VIEWSTATE", "");
				}
				// klic k VIEWSTATE v ulozisti...
				RegisterHiddenField("__VIEWSTATE_KEY", viewStateID.ToString());
			}
			catch (Exception exc) 
			{
				throw new ApplicationException("Chyba pi ukldn ViewState", exc);
			}			
		}

		/// <summary>
		/// pekryt metody pro naten ViewState ze zvolenho loit.
		/// </summary>
		/// <returns>Objekt ve kterm je obsah ViewState</returns>
		protected override object LoadPageStateFromPersistenceMedium() 
		{			
			// zjistime nakonfigurovany typ uloziste
			EViewStateStore configuredStore = Configuration.ViewStateStore;			
			// zjistime nakonfigurovanou dobu expiraci objektu v ulozisti
			int configuredExpiration = Configuration.ObjectExpiration;
			BasePersistor usedPersistor;
			// podle typu konfigurovaneho uloziste vytvorime instanci
			switch (configuredStore) 
			{
				case EViewStateStore.Cache:					
					usedPersistor = new CachePersistor(Cache, configuredExpiration);
					break;
				case EViewStateStore.Database:
					string configuredConnectionString = Configuration.ConnectionString;
					usedPersistor = new DatabasePersistor(configuredConnectionString, configuredExpiration);
					break;
				case EViewStateStore.File:
					string configuredDirectory = Configuration.Directory;
					usedPersistor = new FilePersistor(configuredDirectory, configuredExpiration);
					break;
				default:
					return base.LoadPageStateFromPersistenceMedium();										
			}						
			try 
			{
				// zjistime klic k ulozenemu ViewState ulozeny ve strance
				Guid viewStateID = new Guid(Request.Form["__VIEWSTATE_KEY"]);
				// nacteme ViewState z uloziste
				return usedPersistor.GetViewState(
					viewStateID);
			}
			catch (Exception exc) 
			{				
				// throw new ApplicationException("Chyba pi natn ViewState", exc);
				return new Triplet(string.Empty, string.Empty, string.Empty);
			}			
		}

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);
		}
		#endregion
	}
}
