using System;
using System.Web.UI;
using System.IO;
using System.Text;

namespace ViewState
{
	/// <summary> 
	/// BaseVSDALC je bzovou abstraktn tdou pro tdy implementujc ukldn ViewState do specifickho loit.
	/// </summary> 
	public abstract class BasePersistor 
	{ 
		protected int mObjectExpirationTimeout;

		#region constructors
		public BasePersistor(int objectExpirationTimeout) 
		{
			mObjectExpirationTimeout = objectExpirationTimeout;
		}
		#endregion

		#region public methods
		/// <summary>
		/// Abstraktn metoda pro natn ViewState z loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor natanho ViewState</param>
		/// <returns>Naten ViewState.</returns>
		public abstract object GetViewState(Guid viewStateID);

		/// <summary>
		/// Abstraktn metoda pro ukldn ViewState do loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor ViewState</param>
		/// <param name="viewStateBag">Objekt obsahujc ukldan ViewState</param>
		public abstract void SaveViewState(Guid viewStateID, object viewStateBag);

		/// <summary>
		/// Abstraktn metoda pro vymazn vech uloench ViewState z loit.
		/// </summary>
		public abstract void DeleteApplicationViewStates();
		

		/// <summary>
		/// Abstraktn metoda pro vymazn vech poloek ViewState s vyprenou platnost z loit.
		/// </summary>
		public abstract void DeleteExpiredViewStates();
		#endregion

		#region private and protected methods
		protected object SerializeViewState(object originalViewState) 
		{
			LosFormatter formatter = new LosFormatter();
			StringBuilder stringBuilder = new StringBuilder();
			StringWriter stringWriter = new StringWriter(stringBuilder);
			formatter.Serialize(stringWriter, originalViewState);
			return stringBuilder.ToString();
		}

		protected void SerializeViewState(object originalViewState, TextWriter writer) 
		{
			LosFormatter formatter = new LosFormatter();						
			formatter.Serialize(writer, originalViewState);			
		}

		protected object DeserializeViewState(object serializedObject) 
		{
			object deserializedViewState;				
			LosFormatter formatter = new LosFormatter();
			try 
			{
				deserializedViewState = formatter.Deserialize((string) serializedObject);
				return deserializedViewState;
			}
			catch (Exception exc) 
			{
				throw exc;
			}
		}		
		#endregion
	} 
}
