using System;
using System.Web.Caching;

namespace ViewState
{
	/// <summary>
	/// Tda CacheVSDALC implementuje loit ViewState v cache serveru. 
	/// </summary>
	public class CachePersistor : BasePersistor
	{
		#region private members
		private Cache mPageCache;		
		#endregion

		#region constructors
		public CachePersistor(Cache pageCache, int objectExpirationTimeout) : base(objectExpirationTimeout)
		{			
			mPageCache = pageCache;			
		}
		#endregion

		#region public methods
		/// <summary>
		/// Metoda pro natn ViewState z loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor natanho ViewState</param>
		/// <returns>Naten ViewState.</returns>
		public override object GetViewState(Guid viewStateID) 
		{
			// zjitn kle k objektu v Cache
			string keyValue = GetKey(viewStateID);			
			try 
			{
				// naten objektu z cache
				object serializedViewStateBag = mPageCache.Get(keyValue);
				// deserializace objektu
				object deserializedViewStateBag = DeserializeViewState(serializedViewStateBag);
				// vrcen ViewState
				return deserializedViewStateBag;
			}
			catch (Exception exc) 
			{
				throw new Exception("Chyba pi natn ViewState z cache.", exc);
			}								
		}

		/// <summary>
		/// Metoda pro ukldn ViewState do loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor ViewState</param>
		/// <param name="viewStateBag">Objekt obsahujc ukldan ViewState</param>
		public override void SaveViewState(Guid viewStateID, object viewStateBag) 
		{	
			// zjitn kle k objektu v Cache
			string keyValue = GetKey(viewStateID);				
			try 
			{
				// serializace ViewState
				object serializedViewStateBag = SerializeViewState(viewStateBag);
				// zjistme jestli poloku v Cache pepisujeme nebo vkldme novou
				if (mPageCache.Get(keyValue) == null) 
				{
					// vlozeni nove polozky do cache
					mPageCache.Insert(keyValue, serializedViewStateBag, null, Cache.NoAbsoluteExpiration, TimeSpan.FromSeconds(mObjectExpirationTimeout));
				}
				else 
				{
					// aktualizace stavajici polozky v cache
					mPageCache[keyValue] = serializedViewStateBag;
				}
			}
			catch (Exception exc) 
			{
				throw new Exception("Chyba pi ukldn ViewState do cache.", exc);
			}
		}

		/// <summary>
		/// Metoda pro vymazn vech uloench ViewState z loit.
		/// </summary>
		public override void DeleteApplicationViewStates() 
		{
			// z cache se objekty smazou po uplynuti sepcifikovane casove periody
			return;
		}

		/// <summary>
		/// Metoda pro vymazn vech poloek ViewState s vyprenou platnost z loit.
		/// </summary>
		public override void DeleteExpiredViewStates() 
		{
			// poloky z pamov cache jsou mazny automaticky
			return;
		}
		#endregion

		#region private and protected methods
		protected virtual string GetKey(Guid viewStateID) 
		{			
			string keyValue = "VIEW_STATE_" + viewStateID.ToString();
			return keyValue;
		}		
		#endregion
	}
}
