using System;
using System.Collections.Specialized;

namespace ViewState
{
	/// <summary>
	/// Tda Configuration zodpovd za prci s konfiguranmi daji.
	/// </summary>
	public class Configuration
	{
		private const int defaultObjectExpiration = 1200;
		private const int defaultDeleteExpiredPeriod = 600;
			

		private Configuration() {}		

		/// <summary>
		/// Vlastnost specifikuje typ uloen pole ViewState pro strnky. Pokud nen nastavena, pouv se defaultn implementace ukldn ViewState do skrytho pole v generovan HTML strnce.
		/// </summary>
		public static EViewStateStore ViewStateStore 
		{
			get { 				
				EViewStateStore configuredStateStoreType = EViewStateStore.HiddenField;
				try 
				{
					configuredStateStoreType = (EViewStateStore) Enum.Parse(typeof(EViewStateStore), GetConfigValue("Type")); 													
				}
				catch 
				{					
					configuredStateStoreType = EViewStateStore.HiddenField;
				}
				return configuredStateStoreType;
			}
		}

		/// <summary>
		/// Vlastnost specifikuje pouit pipojovac etzec pro uloen ViewState do databze, pouije se pouze v ppad, e ViewStateStore je nastavena na typ Database (1).
		/// </summary>
		public static string ConnectionString 
		{
			get { 
				try 
				{
					return GetConfigValue("ConnectionString"); 								
				}
				catch 
				{
					return string.Empty;
				}
			}
		}

		/// <summary>
		/// Vlastnost specifikuje adres, ve kterm budou ukldny soubory s ViewState, pouije se pouze v ppad, e ViewStateStore je nastavena na typ File (2).
		/// </summary>
		public static string Directory 
		{
			get { 
				try 
				{					
					return GetConfigValue("Directory"); 													
				}
				catch 
				{
					return string.Empty;
				}
			}
		}

		/// <summary>
		/// Vlastnost specifikuje timeout pro expiraci ViewState v pamov kei na serveru (timeout se pot od poslednho pstupu k poloce kee), pouije se pouze v ppad, e ViewStateStore je nastavena na typ Cache (3).
		/// </summary>
		public static int ObjectExpiration 
		{
			get { 
				try 
				{					
					return int.Parse(GetConfigValue("ObjectExpiration"));
				}
				catch  
				{
					return defaultObjectExpiration;
				}
			}
		}
		
		public static int DeleteExpiredPeriod 
		{
			get 
			{
				try 
				{					
					return int.Parse(GetConfigValue("DeleteExpiredPeriod"));					
				}
				catch 
				{
					return defaultDeleteExpiredPeriod;
				}
			}
		}

		private static string GetConfigValue(string valueKeyName) 
		{
			NameValueCollection viewStateStore = (NameValueCollection) System.Configuration.ConfigurationSettings.GetConfig("viewStateStore");						
			return viewStateStore[valueKeyName];
		}
	}
}
