using System;
using System.Data;
using System.Data.SqlClient;


namespace ViewState
{
	/// <summary>
	/// Tda ViewStateDALC implementuje databzov loit ViewState.
	/// </summary>
	public class DatabasePersistor : BasePersistor
	{
		#region private members and constants
		private string mConnectionString;

		private const string loadViewState = "vs_get_viewstate";
		private const string saveViewState = "vs_save_viewstate";
		private const string deleteViewStatesForApplication = "vs_delete_viewstate";
		private const string deleteExpiredViewStates = "vs_delete_expired_viewstates";
		#endregion

		#region constructors
		/// <summary>
		/// Konstruktor tdy
		/// </summary>
		/// <param name="connectionString">Pipojovac etzec k databzi ve kter do kter se budou ukldat poloky ViewState</param>
		public DatabasePersistor(string connectionString, int objectExpirationTimeout) : base(objectExpirationTimeout)
		{
			mConnectionString = connectionString;						
		}
		#endregion

		#region public methods
		/// <summary>
		/// Abstraktn metoda pro natn ViewState z loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor natanho ViewState</param>
		/// <returns>Naten ViewState.</returns>
		public override object GetViewState(Guid viewStateID) 
		{
			SqlConnection connection = new SqlConnection(mConnectionString);
			try 
			{
				connection.Open();
				SqlCommand command = new SqlCommand(loadViewState, connection);
				command.CommandType = CommandType.StoredProcedure;
				SqlParameter viewStateIDParameter = command.Parameters.Add("@ViewStateID", SqlDbType.UniqueIdentifier);
				viewStateIDParameter.Value = viewStateID;				
				object serializedViewState = command.ExecuteScalar();
				// deserializace objektu
				try 
				{
					return DeserializeViewState(serializedViewState);
				}
				catch (Exception exc) 
				{
					throw exc;
				}
			}
			finally 
			{
				connection.Close();
			}			
		}


		/// <summary>
		/// Metoda pro ukldn ViewState do loit.
		/// </summary>
		/// <param name="viewStateID">Identifiktor ViewState</param>
		/// <param name="viewStateBag">Objekt obsahujc ukldan ViewState</param>
		public override void SaveViewState(Guid viewStateID, object viewStateBag) 
		{
			SqlConnection connection = new SqlConnection(mConnectionString);
			try 
			{
				connection.Open();
				SqlCommand command = new SqlCommand(saveViewState, connection);
				command.CommandType = CommandType.StoredProcedure;				
				SqlParameter viewStateIDParameter = command.Parameters.Add("@ViewStateID", SqlDbType.UniqueIdentifier);
				viewStateIDParameter.Value = viewStateID;								
				SqlParameter viewStateBagParameter = command.Parameters.Add("@ViewStateBag", SqlDbType.Text);										
				// serializace ViewState do parametru									
				viewStateBagParameter.Value = SerializeViewState(viewStateBag);
				command.ExecuteNonQuery();			
			}
			finally 
			{
				connection.Close();
			}
		}		

		/// <summary>
		/// Metoda pro vymazn vech uloench ViewState z loit.
		/// </summary>
		public override void DeleteApplicationViewStates() 
		{
			SqlConnection connection = new SqlConnection(mConnectionString);
			try 
			{
				connection.Open();
				SqlCommand command = new SqlCommand(deleteViewStatesForApplication, connection);				
				command.CommandType = CommandType.StoredProcedure;
				command.ExecuteNonQuery();
			}
			finally 
			{
				connection.Close();
			}
		}

		/// <summary>
		/// Metoda pro vymazn vech poloek ViewState s vyprenou platnost z loit.
		/// </summary>
		public override void DeleteExpiredViewStates() 
		{			
			SqlConnection connection = new SqlConnection(mConnectionString);
			try 
			{
				connection.Open();
				SqlCommand command = new SqlCommand(deleteExpiredViewStates, connection);
				command.CommandType = CommandType.StoredProcedure;
				SqlParameter expiredDateTimeParameter = command.Parameters.Add("@ExpiredDateTime", SqlDbType.DateTime);
				// expiredDateTimeParameter.Value = ASD.Functions.ViewState.GetExpirationDateTime();
				expiredDateTimeParameter.Value = 1;
				command.ExecuteNonQuery();
			}
			finally 
			{
				connection.Close();
			}
		}
		#endregion
	}
}
